@@ -21,7 +21,6 @@ class WP_Term_Query {
 	 * SQL string used to perform database query.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 * @var string
 	 */
 	public $request;
@@ -30,7 +29,6 @@ class WP_Term_Query {
 	 * Metadata query container.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 * @var object WP_Meta_Query
 	 */
 	public $meta_query = false;
@@ -39,7 +37,6 @@ class WP_Term_Query {
 	 * Metadata query clauses.
 	 *
 	 * @since 4.6.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $meta_query_clauses;
@@ -48,7 +45,6 @@ class WP_Term_Query {
 	 * SQL query clauses.
 	 *
 	 * @since 4.6.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $sql_clauses = array(
@@ -63,7 +59,6 @@ class WP_Term_Query {
 	 * Query vars set by the user.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 * @var array
 	 */
 	public $query_vars;
@@ -72,7 +67,6 @@ class WP_Term_Query {
 	 * Default values for query vars.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 * @var array
 	 */
 	public $query_var_defaults;
@@ -81,7 +75,6 @@ class WP_Term_Query {
 	 * List of terms located by the query.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 * @var array
 	 */
 	public $terms;
@@ -93,17 +86,21 @@ class WP_Term_Query {
 	 *
 	 * @since 4.6.0
 	 * @since 4.6.0 Introduced 'term_taxonomy_id' parameter.
-	 * @access public
+	 * @since 4.7.0 Introduced 'object_ids' parameter.
+	 * @since 4.9.0 Added 'slug__in' support for 'orderby'.
 	 *
 	 * @param string|array $query {
 	 *     Optional. Array or query string of term query parameters. Default empty.
 	 *
 	 *     @type string|array $taxonomy               Taxonomy name, or array of taxonomies, to which results should
 	 *                                                be limited.
+	 *     @type int|array    $object_ids             Optional. Object ID, or array of object IDs. Results will be
+	 *                                                limited to terms associated with these objects.
 	 *     @type string       $orderby                Field(s) to order terms by. Accepts term fields ('name',
-	 *                                                'slug', 'term_group', 'term_id', 'id', 'description'),
+	 *                                                'slug', 'term_group', 'term_id', 'id', 'description', 'parent'),
 	 *                                                'count' for term taxonomy count, 'include' to match the
-	 *                                                'order' of the $include param, 'meta_value', 'meta_value_num',
+	 *                                                'order' of the $include param, 'slug__in' to match the
+	 *                                                'order' of the $slug param, 'meta_value', 'meta_value_num',
 	 *                                                the value of `$meta_key`, the array keys of `$meta_query`, or
 	 *                                                'none' to omit the ORDER BY clause. Defaults to 'name'.
 	 *     @type string       $order                  Whether to order terms in ascending or descending order.
@@ -120,16 +117,22 @@ class WP_Term_Query {
 	 *                                                along with all of their descendant terms. If $include is
 	 *                                                non-empty, $exclude_tree is ignored. Default empty array.
 	 *     @type int|string   $number                 Maximum number of terms to return. Accepts ''|0 (all) or any
-	 *                                                positive number. Default ''|0 (all).
+	 *                                                positive number. Default ''|0 (all). Note that $number may
+	 *                                                not return accurate results when coupled with $object_ids.
+	 *                                                See #41796 for details.
 	 *     @type int          $offset                 The number by which to offset the terms query. Default empty.
 	 *     @type string       $fields                 Term fields to query for. Accepts 'all' (returns an array of
-	 *                                                complete term objects), 'ids' (returns an array of ids),
-	 *                                                'id=>parent' (returns an associative array with ids as keys,
-	 *                                                parent term IDs as values), 'names' (returns an array of term
-	 *                                                names), 'count' (returns the number of matching terms),
-	 *                                                'id=>name' (returns an associative array with ids as keys,
-	 *                                                term names as values), or 'id=>slug' (returns an associative
-	 *                                                array with ids as keys, term slugs as values). Default 'all'.
+	 *                                                complete term objects), 'all_with_object_id' (returns an
+	 *                                                array of term objects with the 'object_id' param; only works
+	 *                                                when the `$fields` parameter is 'object_ids' ), 'ids'
+	 *                                                (returns an array of ids), 'tt_ids' (returns an array of
+	 *                                                term taxonomy ids), 'id=>parent' (returns an associative
+	 *                                                array with ids as keys, parent term IDs as values), 'names'
+	 *                                                (returns an array of term names), 'count' (returns the number
+	 *                                                of matching terms), 'id=>name' (returns an associative array
+	 *                                                with ids as keys, term names as values), or 'id=>slug'
+	 *                                                (returns an associative array with ids as keys, term slugs
+	 *                                                as values). Default 'all'.
 	 *     @type bool         $count                  Whether to return a term count (true) or array of term objects
 	 *                                                (false). Will take precedence over `$fields` if true.
 	 *                                                Default false.
@@ -166,14 +169,18 @@ class WP_Term_Query {
 	 *     @type array        $meta_query             Optional. Meta query clauses to limit retrieved terms by.
 	 *                                                See `WP_Meta_Query`. Default empty.
 	 *     @type string       $meta_key               Limit terms to those matching a specific metadata key.
-	 *                                                Can be used in conjunction with `$meta_value`.
+	 *                                                Can be used in conjunction with `$meta_value`. Default empty.
 	 *     @type string       $meta_value             Limit terms to those matching a specific metadata value.
-	 *                                                Usually used in conjunction with `$meta_key`.
+	 *                                                Usually used in conjunction with `$meta_key`. Default empty.
+	 *     @type string       $meta_type              Type of object metadata is for (e.g., comment, post, or user).
+	 *                                                Default empty.
+	 *     @type string       $meta_compare           Comparison operator to test the 'meta_value'. Default empty.
 	 * }
 	 */
 	public function __construct( $query = '' ) {
 		$this->query_var_defaults = array(
 			'taxonomy'               => null,
+			'object_ids'             => null,
 			'orderby'                => 'name',
 			'order'                  => 'ASC',
 			'hide_empty'             => true,
@@ -199,6 +206,10 @@ class WP_Term_Query {
 			'cache_domain'           => 'core',
 			'update_term_meta_cache' => true,
 			'meta_query'             => '',
+			'meta_key'               => '',
+			'meta_value'             => '',
+			'meta_type'              => '',
+			'meta_compare'           => '',
 		);
 
 		if ( ! empty( $query ) ) {
@@ -210,7 +221,6 @@ class WP_Term_Query {
 	 * Parse arguments passed to the term query with default query parameters.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 *
 	 * @param string|array $query WP_Term_Query arguments. See WP_Term_Query::__construct()
 	 */
@@ -253,15 +263,7 @@ class WP_Term_Query {
 
 		$query['taxonomy'] = $taxonomies;
 
-		/**
-		 * Filters the terms query arguments.
-		 *
-		 * @since 3.1.0
-		 *
-		 * @param array $args       An array of get_terms() arguments.
-		 * @param array $taxonomies An array of taxonomies.
-		 */
-		$this->query_vars = apply_filters( 'get_terms_args', $query, $taxonomies );
+		$this->query_vars = $query;
 
 		/**
 		 * Fires after term query vars have been parsed.
@@ -277,7 +279,6 @@ class WP_Term_Query {
 	 * Sets up the query for retrieving terms.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 *
 	 * @param string|array $query Array or URL query string of parameters.
 	 * @return array|int List of terms, or number of terms when 'count' is passed as a query var.
@@ -290,18 +291,17 @@ class WP_Term_Query {
 	/**
 	 * Get terms, based on query_vars.
 	 *
-	 * @param 4.6.0
-	 * @access public
+	 * @since 4.6.0
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
-	 * @return array
+	 * @return array List of terms.
 	 */
 	public function get_terms() {
 		global $wpdb;
 
 		$this->parse_query( $this->query_vars );
-		$args = $this->query_vars;
+		$args = &$this->query_vars;
 
 		// Set up meta_query so it's available to 'pre_get_terms'.
 		$this->meta_query = new WP_Meta_Query();
@@ -316,7 +316,7 @@ class WP_Term_Query {
 		 */
 		do_action( 'pre_get_terms', $this );
 
-		$taxonomies = $args['taxonomy'];
+		$taxonomies = (array) $args['taxonomy'];
 
 		// Save queries by not crawling the tree in the case of multiple taxes or a flat tax.
 		$has_hierarchical_tax = false;
@@ -383,7 +383,13 @@ class WP_Term_Query {
 			}
 		}
 
-		$orderby = $this->parse_orderby( $this->query_vars['orderby'] );
+		// 'term_order' is a legal sort order only when joining the relationship table.
+		$_orderby = $this->query_vars['orderby'];
+		if ( 'term_order' === $_orderby && empty( $this->query_vars['object_ids'] ) ) {
+			$_orderby = 'term_id';
+		}
+		$orderby = $this->parse_orderby( $_orderby );
+
 		if ( $orderby ) {
 			$orderby = "ORDER BY $orderby";
 		}
@@ -416,7 +422,7 @@ class WP_Term_Query {
 			foreach ( $exclude_tree as $extrunk ) {
 				$excluded_children = array_merge(
 					$excluded_children,
-					(array) get_terms( $taxonomies[0], array(
+					(array) get_terms( reset( $taxonomies ), array(
 						'child_of' => intval( $extrunk ),
 						'fields' => 'ids',
 						'hide_empty' => 0
@@ -461,7 +467,10 @@ class WP_Term_Query {
 			$this->sql_clauses['where']['exclusions'] = preg_replace( '/^\s*AND\s*/', '', $exclusions );
 		}
 
-		if ( ! empty( $args['name'] ) ) {
+		if (
+			( ! empty( $args['name'] ) ) ||
+			( is_string( $args['name'] ) && 0 !== strlen( $args['name'] ) )
+		) {
 			$names = (array) $args['name'];
 			foreach ( $names as &$_name ) {
 				// `sanitize_term_field()` returns slashed data.
@@ -471,7 +480,10 @@ class WP_Term_Query {
 			$this->sql_clauses['where']['name'] = "t.name IN ('" . implode( "', '", array_map( 'esc_sql', $names ) ) . "')";
 		}
 
-		if ( ! empty( $args['slug'] ) ) {
+		if (
+			( ! empty( $args['slug'] ) ) ||
+			( is_string( $args['slug'] ) && 0 !== strlen( $args['slug'] ) )
+		) {
 			if ( is_array( $args['slug'] ) ) {
 				$slug = array_map( 'sanitize_title', $args['slug'] );
 				$this->sql_clauses['where']['slug'] = "t.slug IN ('" . implode( "', '", $slug ) . "')";
@@ -498,6 +510,24 @@ class WP_Term_Query {
 			$this->sql_clauses['where']['description__like'] = $wpdb->prepare( "tt.description LIKE %s", '%' . $wpdb->esc_like( $args['description__like'] ) . '%' );
 		}
 
+		if ( ! empty( $args['object_ids'] ) ) {
+			$object_ids = $args['object_ids'];
+			if ( ! is_array( $object_ids ) ) {
+				$object_ids = array( $object_ids );
+			}
+
+			$object_ids = implode( ', ', array_map( 'intval', $object_ids ) );
+			$this->sql_clauses['where']['object_ids'] = "tr.object_id IN ($object_ids)";
+		}
+
+		/*
+		 * When querying for object relationships, the 'count > 0' check
+		 * added by 'hide_empty' is superfluous.
+		 */
+		if ( ! empty( $args['object_ids'] ) ) {
+			$args['hide_empty'] = false;
+		}
+
 		if ( '' !== $parent ) {
 			$parent = (int) $parent;
 			$this->sql_clauses['where']['parent'] = "tt.parent = '$parent'";
@@ -549,7 +579,13 @@ class WP_Term_Query {
 		$selects = array();
 		switch ( $args['fields'] ) {
 			case 'all':
+			case 'all_with_object_id' :
+			case 'tt_ids' :
+			case 'slugs' :
 				$selects = array( 't.*', 'tt.*' );
+				if ( 'all_with_object_id' === $args['fields'] && ! empty( $args['object_ids'] ) ) {
+					$selects[] = 'tr.object_id';
+				}
 				break;
 			case 'ids':
 			case 'id=>parent':
@@ -593,9 +629,11 @@ class WP_Term_Query {
 
 		$join .= " INNER JOIN $wpdb->term_taxonomy AS tt ON t.term_id = tt.term_id";
 
-		$where = implode( ' AND ', $this->sql_clauses['where'] );
+		if ( ! empty( $this->query_vars['object_ids'] ) ) {
+			$join .= " INNER JOIN {$wpdb->term_relationships} AS tr ON tr.term_taxonomy_id = tt.term_taxonomy_id";
+		}
 
-		$pieces = array( 'fields', 'join', 'where', 'distinct', 'orderby', 'order', 'limits' );
+		$where = implode( ' AND ', $this->sql_clauses['where'] );
 
 		/**
 		 * Filters the terms query SQL clauses.
@@ -606,7 +644,7 @@ class WP_Term_Query {
 		 * @param array $taxonomies An array of taxonomies.
 		 * @param array $args       An array of terms query arguments.
 		 */
-		$clauses = apply_filters( 'terms_clauses', compact( $pieces ), $taxonomies, $args );
+		$clauses = apply_filters( 'terms_clauses', compact( 'fields', 'join', 'where', 'distinct', 'orderby', 'order', 'limits' ), $taxonomies, $args );
 
 		$fields = isset( $clauses[ 'fields' ] ) ? $clauses[ 'fields' ] : '';
 		$join = isset( $clauses[ 'join' ] ) ? $clauses[ 'join' ] : '';
@@ -629,15 +667,11 @@ class WP_Term_Query {
 
 		// $args can be anything. Only use the args defined in defaults to compute the key.
 		$key = md5( serialize( wp_array_slice_assoc( $args, array_keys( $this->query_var_defaults ) ) ) . serialize( $taxonomies ) . $this->request );
-		$last_changed = wp_cache_get( 'last_changed', 'terms' );
-		if ( ! $last_changed ) {
-			$last_changed = microtime();
-			wp_cache_set( 'last_changed', $last_changed, 'terms' );
-		}
+		$last_changed = wp_cache_get_last_changed( 'terms' );
 		$cache_key = "get_terms:$key:$last_changed";
 		$cache = wp_cache_get( $cache_key, 'terms' );
 		if ( false !== $cache ) {
-			if ( 'all' === $_fields ) {
+			if ( 'all' === $_fields || 'all_with_object_id' === $_fields ) {
 				$cache = array_map( 'get_term', $cache );
 			}
 
@@ -646,11 +680,13 @@ class WP_Term_Query {
 		}
 
 		if ( 'count' == $_fields ) {
-			return $wpdb->get_var( $this->request );
+			$count = $wpdb->get_var( $this->request );
+			wp_cache_set( $cache_key, $count, 'terms' );
+			return $count;
 		}
 
 		$terms = $wpdb->get_results( $this->request );
-		if ( 'all' == $_fields ) {
+		if ( 'all' == $_fields || 'all_with_object_id' === $_fields ) {
 			update_term_cache( $terms );
 		}
 
@@ -701,6 +737,26 @@ class WP_Term_Query {
 			}
 		}
 
+		/*
+		 * When querying for terms connected to objects, we may get
+		 * duplicate results. The duplicates should be preserved if
+		 * `$fields` is 'all_with_object_id', but should otherwise be
+		 * removed.
+		 */
+		if ( ! empty( $args['object_ids'] ) && 'all_with_object_id' != $_fields ) {
+			$_tt_ids = $_terms = array();
+			foreach ( $terms as $term ) {
+				if ( isset( $_tt_ids[ $term->term_id ] ) ) {
+					continue;
+				}
+
+				$_tt_ids[ $term->term_id ] = 1;
+				$_terms[] = $term;
+			}
+
+			$terms = $_terms;
+		}
+
 		$_terms = array();
 		if ( 'id=>parent' == $_fields ) {
 			foreach ( $terms as $term ) {
@@ -708,12 +764,20 @@ class WP_Term_Query {
 			}
 		} elseif ( 'ids' == $_fields ) {
 			foreach ( $terms as $term ) {
-				$_terms[] = $term->term_id;
+				$_terms[] = (int) $term->term_id;
+			}
+		} elseif ( 'tt_ids' == $_fields ) {
+			foreach ( $terms as $term ) {
+				$_terms[] = (int) $term->term_taxonomy_id;
 			}
 		} elseif ( 'names' == $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[] = $term->name;
 			}
+		} elseif ( 'slugs' == $_fields ) {
+			foreach ( $terms as $term ) {
+				$_terms[] = $term->slug;
+			}
 		} elseif ( 'id=>name' == $_fields ) {
 			foreach ( $terms as $term ) {
 				$_terms[ $term->term_id ] = $term->name;
@@ -739,7 +803,7 @@ class WP_Term_Query {
 
 		wp_cache_add( $cache_key, $terms, 'terms', DAY_IN_SECONDS );
 
-		if ( 'all' === $_fields ) {
+		if ( 'all' === $_fields || 'all_with_object_id' === $_fields ) {
 			$terms = array_map( 'get_term', $terms );
 		}
 
@@ -751,7 +815,6 @@ class WP_Term_Query {
 	 * Parse and sanitize 'orderby' keys passed to the term query.
 	 *
 	 * @since 4.6.0
-	 * @access protected
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
@@ -761,19 +824,19 @@ class WP_Term_Query {
 	protected function parse_orderby( $orderby_raw ) {
 		$_orderby = strtolower( $orderby_raw );
 		$maybe_orderby_meta = false;
-		if ( 'count' == $_orderby ) {
-			$orderby = 'tt.count';
-		} elseif ( 'name' == $_orderby ) {
-			$orderby = 't.name';
-		} elseif ( 'slug' == $_orderby ) {
-			$orderby = 't.slug';
+
+		if ( in_array( $_orderby, array( 'term_id', 'name', 'slug', 'term_group' ), true ) ) {
+			$orderby = "t.$_orderby";
+		} elseif ( in_array( $_orderby, array( 'count', 'parent', 'taxonomy', 'term_taxonomy_id', 'description' ), true ) ) {
+			$orderby = "tt.$_orderby";
+		} elseif ( 'term_order' === $_orderby ) {
+			$orderby = 'tr.term_order';
 		} elseif ( 'include' == $_orderby && ! empty( $this->query_vars['include'] ) ) {
-			$include = implode( ',', array_map( 'absint', $this->query_vars['include'] ) );
+			$include = implode( ',', wp_parse_id_list( $this->query_vars['include'] ) );
 			$orderby = "FIELD( t.term_id, $include )";
-		} elseif ( 'term_group' == $_orderby ) {
-			$orderby = 't.term_group';
-		} elseif ( 'description' == $_orderby ) {
-			$orderby = 'tt.description';
+		} elseif ( 'slug__in' == $_orderby && ! empty( $this->query_vars['slug'] ) && is_array( $this->query_vars['slug'] ) ) {
+			$slugs = implode( "', '", array_map( 'sanitize_title_for_query', $this->query_vars['slug'] ) );
+			$orderby = "FIELD( t.slug, '" . $slugs . "')";
 		} elseif ( 'none' == $_orderby ) {
 			$orderby = '';
 		} elseif ( empty( $_orderby ) || 'id' == $_orderby || 'term_id' === $_orderby ) {
@@ -811,10 +874,9 @@ class WP_Term_Query {
 	 * Generate the ORDER BY clause for an 'orderby' param that is potentially related to a meta query.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 *
 	 * @param string $orderby_raw Raw 'orderby' value passed to WP_Term_Query.
-	 * @return string
+	 * @return string ORDER BY clause.
 	 */
 	protected function parse_orderby_meta( $orderby_raw ) {
 		$orderby = '';
@@ -871,7 +933,6 @@ class WP_Term_Query {
 	 * Parse an 'order' query variable and cast it to ASC or DESC as necessary.
 	 *
 	 * @since 4.6.0
-	 * @access protected
 	 *
 	 * @param string $order The 'order' query variable.
 	 * @return string The sanitized 'order' query variable.
@@ -892,7 +953,6 @@ class WP_Term_Query {
 	 * Used internally to generate a SQL string related to the 'search' parameter.
 	 *
 	 * @since 4.6.0
-	 * @access protected
 	 *
 	 * @global wpdb $wpdb WordPress database abstraction object.
 	 *
