@@ -45,8 +45,6 @@ define( 'ARRAY_N', 'ARRAY_N' );
  *
  * @link https://codex.wordpress.org/Function_Reference/wpdb_Class
  *
- * @package WordPress
- * @subpackage Database
  * @since 0.71
  */
 class wpdb {
@@ -58,7 +56,6 @@ class wpdb {
 	 * evaluated to true.
 	 *
 	 * @since 0.71
-	 * @access private
 	 * @var bool
 	 */
 	var $show_errors = false;
@@ -66,7 +63,6 @@ class wpdb {
 	/**
 	 * Whether to suppress errors during the DB bootstrapping.
 	 *
-	 * @access private
 	 * @since 2.5.0
 	 * @var bool
 	 */
@@ -84,7 +80,6 @@ class wpdb {
 	 * Amount of queries made
 	 *
 	 * @since 1.2.0
-	 * @access public
 	 * @var int
 	 */
 	public $num_queries = 0;
@@ -93,7 +88,6 @@ class wpdb {
 	 * Count of rows returned by previous query
 	 *
 	 * @since 0.71
-	 * @access public
 	 * @var int
 	 */
 	public $num_rows = 0;
@@ -102,7 +96,6 @@ class wpdb {
 	 * Count of affected rows by previous query
 	 *
 	 * @since 0.71
-	 * @access private
 	 * @var int
 	 */
 	var $rows_affected = 0;
@@ -111,7 +104,6 @@ class wpdb {
 	 * The ID generated for an AUTO_INCREMENT column by the previous query (usually INSERT).
 	 *
 	 * @since 0.71
-	 * @access public
 	 * @var int
 	 */
 	public $insert_id = 0;
@@ -120,7 +112,6 @@ class wpdb {
 	 * Last query made
 	 *
 	 * @since 0.71
-	 * @access private
 	 * @var array
 	 */
 	var $last_query;
@@ -129,7 +120,6 @@ class wpdb {
 	 * Results of the last query made
 	 *
 	 * @since 0.71
-	 * @access private
 	 * @var array|null
 	 */
 	var $last_result;
@@ -138,7 +128,6 @@ class wpdb {
 	 * MySQL result, which is either a resource or boolean.
 	 *
 	 * @since 0.71
-	 * @access protected
 	 * @var mixed
 	 */
 	protected $result;
@@ -147,7 +136,6 @@ class wpdb {
 	 * Cached column info, for sanity checking data before inserting
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $col_meta = array();
@@ -156,7 +144,6 @@ class wpdb {
 	 * Calculated character sets on tables
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $table_charset = array();
@@ -165,7 +152,6 @@ class wpdb {
 	 * Whether text fields in the current query need to be sanity checked.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 * @var bool
 	 */
 	protected $check_current_query = true;
@@ -174,7 +160,6 @@ class wpdb {
 	 * Flag to ensure we don't run into recursion problems when checking the collation.
 	 *
 	 * @since 4.2.0
-	 * @access private
 	 * @see wpdb::check_safe_collation()
 	 * @var bool
 	 */
@@ -184,7 +169,6 @@ class wpdb {
 	 * Saved info on the table column
 	 *
 	 * @since 0.71
-	 * @access protected
 	 * @var array
 	 */
 	protected $col_info;
@@ -193,7 +177,6 @@ class wpdb {
 	 * Saved queries that were executed
 	 *
 	 * @since 1.5.0
-	 * @access private
 	 * @var array
 	 */
 	var $queries;
@@ -202,7 +185,6 @@ class wpdb {
 	 * The number of times to retry reconnecting before dying.
 	 *
 	 * @since 3.9.0
-	 * @access protected
 	 * @see wpdb::check_connection()
 	 * @var int
 	 */
@@ -216,7 +198,6 @@ class wpdb {
 	 * security precautions.
 	 *
 	 * @since 2.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $prefix = '';
@@ -225,7 +206,6 @@ class wpdb {
 	 * WordPress base table prefix.
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	 public $base_prefix;
@@ -234,7 +214,6 @@ class wpdb {
 	 * Whether the database queries are ready to start executing.
 	 *
 	 * @since 2.3.2
-	 * @access private
 	 * @var bool
 	 */
 	var $ready = false;
@@ -243,7 +222,6 @@ class wpdb {
 	 * Blog ID.
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var int
 	 */
 	public $blogid = 0;
@@ -252,7 +230,6 @@ class wpdb {
 	 * Site ID.
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var int
 	 */
 	public $siteid = 0;
@@ -261,7 +238,6 @@ class wpdb {
 	 * List of WordPress per-blog tables
 	 *
 	 * @since 2.5.0
-	 * @access private
 	 * @see wpdb::tables()
 	 * @var array
 	 */
@@ -274,7 +250,6 @@ class wpdb {
 	 * categories, post2cat, and link2cat were deprecated in 2.3.0, db version 5539
 	 *
 	 * @since 2.9.0
-	 * @access private
 	 * @see wpdb::tables()
 	 * @var array
 	 */
@@ -284,7 +259,6 @@ class wpdb {
 	 * List of WordPress global tables
 	 *
 	 * @since 3.0.0
-	 * @access private
 	 * @see wpdb::tables()
 	 * @var array
 	 */
@@ -294,7 +268,6 @@ class wpdb {
 	 * List of Multisite global tables
 	 *
 	 * @since 3.0.0
-	 * @access private
 	 * @see wpdb::tables()
 	 * @var array
 	 */
@@ -305,7 +278,6 @@ class wpdb {
 	 * WordPress Comments table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $comments;
@@ -314,7 +286,6 @@ class wpdb {
 	 * WordPress Comment Metadata table
 	 *
 	 * @since 2.9.0
-	 * @access public
 	 * @var string
 	 */
 	public $commentmeta;
@@ -323,7 +294,6 @@ class wpdb {
 	 * WordPress Links table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $links;
@@ -332,7 +302,6 @@ class wpdb {
 	 * WordPress Options table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $options;
@@ -341,7 +310,6 @@ class wpdb {
 	 * WordPress Post Metadata table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $postmeta;
@@ -350,7 +318,6 @@ class wpdb {
 	 * WordPress Posts table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $posts;
@@ -359,7 +326,6 @@ class wpdb {
 	 * WordPress Terms table
 	 *
 	 * @since 2.3.0
-	 * @access public
 	 * @var string
 	 */
 	public $terms;
@@ -368,7 +334,6 @@ class wpdb {
 	 * WordPress Term Relationships table
 	 *
 	 * @since 2.3.0
-	 * @access public
 	 * @var string
 	 */
 	public $term_relationships;
@@ -377,7 +342,6 @@ class wpdb {
 	 * WordPress Term Taxonomy table
 	 *
 	 * @since 2.3.0
-	 * @access public
 	 * @var string
 	 */
 	public $term_taxonomy;
@@ -386,7 +350,6 @@ class wpdb {
 	 * WordPress Term Meta table.
 	 *
 	 * @since 4.4.0
-	 * @access public
 	 * @var string
 	 */
 	public $termmeta;
@@ -399,7 +362,6 @@ class wpdb {
 	 * WordPress User Metadata table
 	 *
 	 * @since 2.3.0
-	 * @access public
 	 * @var string
 	 */
 	public $usermeta;
@@ -408,7 +370,6 @@ class wpdb {
 	 * WordPress Users table
 	 *
 	 * @since 1.5.0
-	 * @access public
 	 * @var string
 	 */
 	public $users;
@@ -417,7 +378,6 @@ class wpdb {
 	 * Multisite Blogs table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $blogs;
@@ -426,7 +386,6 @@ class wpdb {
 	 * Multisite Blog Versions table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $blog_versions;
@@ -435,7 +394,6 @@ class wpdb {
 	 * Multisite Registration Log table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $registration_log;
@@ -444,7 +402,6 @@ class wpdb {
 	 * Multisite Signups table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $signups;
@@ -453,7 +410,6 @@ class wpdb {
 	 * Multisite Sites table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $site;
@@ -462,7 +418,6 @@ class wpdb {
 	 * Multisite Sitewide Terms table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $sitecategories;
@@ -471,7 +426,6 @@ class wpdb {
 	 * Multisite Site Metadata table
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $sitemeta;
@@ -487,7 +441,6 @@ class wpdb {
 	 * @see wpdb::update()
 	 * @see wpdb::delete()
 	 * @see wp_set_wpdb_vars()
-	 * @access public
 	 * @var array
 	 */
 	public $field_types = array();
@@ -496,7 +449,6 @@ class wpdb {
 	 * Database table columns charset
 	 *
 	 * @since 2.2.0
-	 * @access public
 	 * @var string
 	 */
 	public $charset;
@@ -505,7 +457,6 @@ class wpdb {
 	 * Database table columns collate
 	 *
 	 * @since 2.2.0
-	 * @access public
 	 * @var string
 	 */
 	public $collate;
@@ -514,7 +465,6 @@ class wpdb {
 	 * Database Username
 	 *
 	 * @since 2.9.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $dbuser;
@@ -523,7 +473,6 @@ class wpdb {
 	 * Database Password
 	 *
 	 * @since 3.1.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $dbpassword;
@@ -532,7 +481,6 @@ class wpdb {
 	 * Database Name
 	 *
 	 * @since 3.1.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $dbname;
@@ -541,7 +489,6 @@ class wpdb {
 	 * Database Host
 	 *
 	 * @since 3.1.0
-	 * @access protected
 	 * @var string
 	 */
 	protected $dbhost;
@@ -550,7 +497,6 @@ class wpdb {
 	 * Database Handle
 	 *
 	 * @since 0.71
-	 * @access protected
 	 * @var string
 	 */
 	protected $dbh;
@@ -559,7 +505,6 @@ class wpdb {
 	 * A textual description of the last query/get_row/get_var call
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 * @var string
 	 */
 	public $func_call;
@@ -573,7 +518,6 @@ class wpdb {
 	 * will force the checks to occur.
 	 *
 	 * @since 3.3.0
-	 * @access public
 	 * @var bool
 	 */
 	public $is_mysql = null;
@@ -582,7 +526,6 @@ class wpdb {
 	 * A list of incompatible SQL modes.
 	 *
 	 * @since 3.9.0
-	 * @access protected
 	 * @var array
 	 */
 	protected $incompatible_modes = array( 'NO_ZERO_DATE', 'ONLY_FULL_GROUP_BY',
@@ -592,7 +535,6 @@ class wpdb {
 	 * Whether to use mysqli over mysql.
 	 *
 	 * @since 3.9.0
-	 * @access private
 	 * @var bool
 	 */
 	private $use_mysqli = false;
@@ -601,7 +543,6 @@ class wpdb {
 	 * Whether we've managed to successfully connect at some point
 	 *
 	 * @since 3.9.0
-	 * @access private
 	 * @var bool
 	 */
 	private $has_connected = false;
@@ -629,19 +570,12 @@ class wpdb {
 		if ( WP_DEBUG && WP_DEBUG_DISPLAY )
 			$this->show_errors();
 
-		/* Use ext/mysqli if it exists and:
-		 *  - WP_USE_EXT_MYSQL is defined as false, or
-		 *  - We are a development version of WordPress, or
-		 *  - We are running PHP 5.5 or greater, or
-		 *  - ext/mysql is not loaded.
-		 */
+		// Use ext/mysqli if it exists unless WP_USE_EXT_MYSQL is defined as true
 		if ( function_exists( 'mysqli_connect' ) ) {
+			$this->use_mysqli = true;
+
 			if ( defined( 'WP_USE_EXT_MYSQL' ) ) {
 				$this->use_mysqli = ! WP_USE_EXT_MYSQL;
-			} elseif ( version_compare( phpversion(), '5.5', '>=' ) || ! function_exists( 'mysql_connect' ) ) {
-				$this->use_mysqli = true;
-			} elseif ( false !== strpos( $GLOBALS['wp_version'], '-' ) ) {
-				$this->use_mysqli = true;
 			}
 		}
 
@@ -764,7 +698,6 @@ class wpdb {
 	 * For example, when able, utf8mb4 should be used instead of utf8.
 	 *
 	 * @since 4.6.0
-	 * @access public
 	 *
 	 * @param string $charset The character set to check.
 	 * @param string $collate The collation to check.
@@ -951,15 +884,15 @@ class wpdb {
 	 * Sets blog id.
 	 *
 	 * @since 3.0.0
-	 * @access public
 	 *
 	 * @param int $blog_id
-	 * @param int $site_id Optional.
+	 * @param int $network_id Optional.
 	 * @return int previous blog id
 	 */
-	public function set_blog_id( $blog_id, $site_id = 0 ) {
-		if ( ! empty( $site_id ) )
-			$this->siteid = $site_id;
+	public function set_blog_id( $blog_id, $network_id = 0 ) {
+		if ( ! empty( $network_id ) ) {
+			$this->siteid = $network_id;
+		}
 
 		$old_blog_id  = $this->blogid;
 		$this->blogid = $blog_id;
@@ -1143,7 +1076,6 @@ class wpdb {
 	 * @deprecated 3.6.0 Use wpdb::prepare()
 	 * @see wpdb::prepare
 	 * @see esc_sql()
-	 * @access private
 	 *
 	 * @param string $string
 	 * @return string
@@ -1160,7 +1092,6 @@ class wpdb {
 	 * @see mysqli_real_escape_string()
 	 * @see mysql_real_escape_string()
 	 * @since 2.8.0
-	 * @access private
 	 *
 	 * @param  string $string to escape
 	 * @return string escaped
@@ -1168,20 +1099,22 @@ class wpdb {
 	function _real_escape( $string ) {
 		if ( $this->dbh ) {
 			if ( $this->use_mysqli ) {
-				return mysqli_real_escape_string( $this->dbh, $string );
+				$escaped = mysqli_real_escape_string( $this->dbh, $string );
 			} else {
-				return mysql_real_escape_string( $string, $this->dbh );
+				$escaped = mysql_real_escape_string( $string, $this->dbh );
 			}
+		} else {
+			$class = get_class( $this );
+			if ( function_exists( '__' ) ) {
+				/* translators: %s: database access abstraction class, usually wpdb or a class extending wpdb */
+				_doing_it_wrong( $class, sprintf( __( '%s must set a database connection for use with escaping.' ), $class ), '3.6.0' );
+			} else {
+				_doing_it_wrong( $class, sprintf( '%s must set a database connection for use with escaping.', $class ), '3.6.0' );
+			}
+			$escaped = addslashes( $string );
 		}
 
-		$class = get_class( $this );
-		if ( function_exists( '__' ) ) {
-			/* translators: %s: database access abstraction class, usually wpdb or a class extending wpdb */
-			_doing_it_wrong( $class, sprintf( __( '%s must set a database connection for use with escaping.' ), $class ), '3.6.0' );
-		} else {
-			_doing_it_wrong( $class, sprintf( '%s must set a database connection for use with escaping.', $class ), '3.6.0' );
-		}
-		return addslashes( $string );
+		return $this->add_placeholder_escape( $escaped );
 	}
 
 	/**
@@ -1189,7 +1122,6 @@ class wpdb {
 	 *
 	 * @uses wpdb::_real_escape()
 	 * @since  2.8.0
-	 * @access public
 	 *
 	 * @param  string|array $data
 	 * @return string|array escaped
@@ -1257,67 +1189,120 @@ class wpdb {
 	/**
 	 * Prepares a SQL query for safe execution. Uses sprintf()-like syntax.
 	 *
-	 * The following directives can be used in the query format string:
+	 * The following placeholders can be used in the query string:
 	 *   %d (integer)
 	 *   %f (float)
 	 *   %s (string)
-	 *   %% (literal percentage sign - no argument needed)
 	 *
-	 * All of %d, %f, and %s are to be left unquoted in the query string and they need an argument passed for them.
-	 * Literals (%) as parts of the query must be properly written as %%.
+	 * All placeholders MUST be left unquoted in the query string. A corresponding argument MUST be passed for each placeholder.
 	 *
-	 * This function only supports a small subset of the sprintf syntax; it only supports %d (integer), %f (float), and %s (string).
-	 * Does not support sign, padding, alignment, width or precision specifiers.
-	 * Does not support argument numbering/swapping.
+	 * For compatibility with old behavior, numbered or formatted string placeholders (eg, %1$s, %5s) will not have quotes
+	 * added by this function, so should be passed with appropriate quotes around them for your usage.
 	 *
-	 * May be called like {@link https://secure.php.net/sprintf sprintf()} or like {@link https://secure.php.net/vsprintf vsprintf()}.
+	 * Literal percentage signs (%) in the query string must be written as %%. Percentage wildcards (for example,
+	 * to use in LIKE syntax) must be passed via a substitution argument containing the complete LIKE string, these
+	 * cannot be inserted directly in the query string. Also see {@see esc_like()}.
 	 *
-	 * Both %d and %s should be left unquoted in the query string.
+	 * Arguments may be passed as individual arguments to the method, or as a single array containing all arguments. A combination
+	 * of the two is not supported.
 	 *
-	 *     $wpdb->prepare( "SELECT * FROM `table` WHERE `column` = %s AND `field` = %d", 'foo', 1337 );
+	 * Examples:
+	 *     $wpdb->prepare( "SELECT * FROM `table` WHERE `column` = %s AND `field` = %d OR `other_field` LIKE %s", array( 'foo', 1337, '%bar' ) );
 	 *     $wpdb->prepare( "SELECT DATE_FORMAT(`field`, '%%c') FROM `table` WHERE `column` = %s", 'foo' );
 	 *
 	 * @link https://secure.php.net/sprintf Description of syntax.
 	 * @since 2.3.0
 	 *
 	 * @param string      $query    Query statement with sprintf()-like placeholders
-	 * @param array|mixed $args     The array of variables to substitute into the query's placeholders if being called like
-	 *                              {@link https://secure.php.net/vsprintf vsprintf()}, or the first variable to substitute into the query's placeholders if
-	 *                              being called like {@link https://secure.php.net/sprintf sprintf()}.
-	 * @param mixed       $args,... further variables to substitute into the query's placeholders if being called like
-	 *                              {@link https://secure.php.net/sprintf sprintf()}.
+	 * @param array|mixed $args     The array of variables to substitute into the query's placeholders if being called with an array of arguments,
+	 *                              or the first variable to substitute into the query's placeholders if being called with individual arguments.
+	 * @param mixed       $args,... further variables to substitute into the query's placeholders if being called wih individual arguments.
 	 * @return string|void Sanitized query string, if there is a query to prepare.
 	 */
 	public function prepare( $query, $args ) {
-		if ( is_null( $query ) )
+		if ( is_null( $query ) ) {
 			return;
+		}
 
 		// This is not meant to be foolproof -- but it will catch obviously incorrect usage.
 		if ( strpos( $query, '%' ) === false ) {
+			wp_load_translations_early();
 			_doing_it_wrong( 'wpdb::prepare', sprintf( __( 'The query argument of %s must have a placeholder.' ), 'wpdb::prepare()' ), '3.9.0' );
 		}
 
 		$args = func_get_args();
 		array_shift( $args );
 
-		// If args were passed as an array (as in vsprintf), move them up
+		// If args were passed as an array (as in vsprintf), move them up.
+		$passed_as_array = false;
 		if ( is_array( $args[0] ) && count( $args ) == 1 ) {
+			$passed_as_array = true;
 			$args = $args[0];
 		}
 
 		foreach ( $args as $arg ) {
 			if ( ! is_scalar( $arg ) && ! is_null( $arg ) ) {
-				_doing_it_wrong( 'wpdb::prepare', sprintf( 'Unsupported value type (%s).', gettype( $arg ) ), '4.8.2' );
+				wp_load_translations_early();
+				_doing_it_wrong( 'wpdb::prepare', sprintf( __( 'Unsupported value type (%s).' ), gettype( $arg ) ), '4.8.2' );
+			}
+		}
+
+		/*
+		 * Specify the formatting allowed in a placeholder. The following are allowed:
+		 *
+		 * - Sign specifier. eg, $+d
+		 * - Numbered placeholders. eg, %1$s
+		 * - Padding specifier, including custom padding characters. eg, %05s, %'#5s
+		 * - Alignment specifier. eg, %05-s
+		 * - Precision specifier. eg, %.2f
+		 */
+		$allowed_format = '(?:[1-9][0-9]*[$])?[-+0-9]*(?: |0|\'.)?[-+0-9]*(?:\.[0-9]+)?';
+
+		/*
+		 * If a %s placeholder already has quotes around it, removing the existing quotes and re-inserting them
+		 * ensures the quotes are consistent.
+		 *
+		 * For backwards compatibility, this is only applied to %s, and not to placeholders like %1$s, which are frequently
+		 * used in the middle of longer strings, or as table name placeholders.
+		 */
+		$query = str_replace( "'%s'", '%s', $query ); // Strip any existing single quotes.
+		$query = str_replace( '"%s"', '%s', $query ); // Strip any existing double quotes.
+		$query = preg_replace( '/(?<!%)%s/', "'%s'", $query ); // Quote the strings, avoiding escaped strings like %%s.
+
+		$query = preg_replace( "/(?<!%)(%($allowed_format)?f)/" , '%\\2F', $query ); // Force floats to be locale unaware.
+
+		$query = preg_replace( "/%(?:%|$|(?!($allowed_format)?[sdF]))/", '%%\\1', $query ); // Escape any unescaped percents.
+
+		// Count the number of valid placeholders in the query.
+		$placeholders = preg_match_all( "/(^|[^%]|(%%)+)%($allowed_format)?[sdF]/", $query, $matches );
+
+		if ( count( $args ) !== $placeholders ) {
+			if ( 1 === $placeholders && $passed_as_array ) {
+				// If the passed query only expected one argument, but the wrong number of arguments were sent as an array, bail.
+				wp_load_translations_early();
+				_doing_it_wrong( 'wpdb::prepare', __( 'The query only expected one placeholder, but an array of multiple placeholders was sent.' ), '4.9.0' );
+
+				return;
+			} else {
+				/*
+				 * If we don't have the right number of placeholders, but they were passed as individual arguments,
+				 * or we were expecting multiple arguments in an array, throw a warning.
+				 */
+				wp_load_translations_early();
+				_doing_it_wrong( 'wpdb::prepare',
+					/* translators: 1: number of placeholders, 2: number of arguments passed */
+					sprintf( __( 'The query does not contain the correct number of placeholders (%1$d) for the number of arguments passed (%2$d).' ),
+						$placeholders,
+						count( $args ) ),
+					'4.8.3'
+				);
 			}
 		}
 
-		$query = str_replace( "'%s'", '%s', $query ); // in case someone mistakenly already singlequoted it
-		$query = str_replace( '"%s"', '%s', $query ); // doublequote unquoting
-		$query = preg_replace( '|(?<!%)%f|' , '%F', $query ); // Force floats to be locale unaware
-		$query = preg_replace( '|(?<!%)%s|', "'%s'", $query ); // quote the strings, avoiding escaped strings like %%s
-		$query = preg_replace( '/%(?:%|$|([^dsF]))/', '%%\\1', $query ); // escape any unescaped percents 
 		array_walk( $args, array( $this, 'escape_by_ref' ) );
-		return @vsprintf( $query, $args );
+		$query = @vsprintf( $query, $args );
+
+		return $this->add_placeholder_escape( $query );
 	}
 
 	/**
@@ -1330,14 +1315,13 @@ class wpdb {
 	 *     $wild = '%';
 	 *     $find = 'only 43% of planets';
 	 *     $like = $wild . $wpdb->esc_like( $find ) . $wild;
-	 *     $sql  = $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE post_content LIKE '%s'", $like );
+	 *     $sql  = $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE post_content LIKE %s", $like );
 	 *
 	 * Example Escape Chain:
 	 *
 	 *     $sql  = esc_sql( $wpdb->esc_like( $input ) );
 	 *
 	 * @since 4.0.0
-	 * @access public
 	 *
 	 * @param string $text The raw text to be escaped. The input typed by the user should have no
 	 *                     extra or deleted slashes.
@@ -1524,24 +1508,23 @@ class wpdb {
 		if ( $this->use_mysqli ) {
 			$this->dbh = mysqli_init();
 
-			// mysqli_real_connect doesn't support the host param including a port or socket
-			// like mysql_connect does. This duplicates how mysql_connect detects a port and/or socket file.
-			$port = null;
-			$socket = null;
-			$host = $this->dbhost;
-			$port_or_socket = strstr( $host, ':' );
-			if ( ! empty( $port_or_socket ) ) {
-				$host = substr( $host, 0, strpos( $host, ':' ) );
-				$port_or_socket = substr( $port_or_socket, 1 );
-				if ( 0 !== strpos( $port_or_socket, '/' ) ) {
-					$port = intval( $port_or_socket );
-					$maybe_socket = strstr( $port_or_socket, ':' );
-					if ( ! empty( $maybe_socket ) ) {
-						$socket = substr( $maybe_socket, 1 );
-					}
-				} else {
-					$socket = $port_or_socket;
-				}
+			$host    = $this->dbhost;
+			$port    = null;
+			$socket  = null;
+			$is_ipv6 = false;
+
+			if ( $host_data = $this->parse_db_host( $this->dbhost ) ) {
+				list( $host, $port, $socket, $is_ipv6 ) = $host_data;
+			}
+
+			/*
+			 * If using the `mysqlnd` library, the IPv6 address needs to be
+			 * enclosed in square brackets, whereas it doesn't while using the
+			 * `libmysqlclient` library.
+			 * @see https://bugs.php.net/bug.php?id=67563
+			 */
+			if ( $is_ipv6 && extension_loaded( 'mysqlnd' ) ) {
+				$host = "[$host]";
 			}
 
 			if ( WP_DEBUG ) {
@@ -1553,7 +1536,8 @@ class wpdb {
 			if ( $this->dbh->connect_errno ) {
 				$this->dbh = null;
 
-				/* It's possible ext/mysqli is misconfigured. Fall back to ext/mysql if:
+				/*
+				 * It's possible ext/mysqli is misconfigured. Fall back to ext/mysql if:
 		 		 *  - We haven't previously connected, and
 		 		 *  - WP_USE_EXT_MYSQL isn't set to false, and
 		 		 *  - ext/mysql is loaded.
@@ -1633,6 +1617,60 @@ class wpdb {
 		return false;
 	}
 
+	/**
+	 * Parse the DB_HOST setting to interpret it for mysqli_real_connect.
+	 *
+	 * mysqli_real_connect doesn't support the host param including a port or
+	 * socket like mysql_connect does. This duplicates how mysql_connect detects
+	 * a port and/or socket file.
+	 *
+	 * @since 4.9.0
+	 *
+	 * @param string $host The DB_HOST setting to parse.
+	 * @return array|bool Array containing the host, the port, the socket and whether
+	 *                    it is an IPv6 address, in that order. If $host couldn't be parsed,
+	 *                    returns false.
+	 */
+	public function parse_db_host( $host ) {
+		$port    = null;
+		$socket  = null;
+		$is_ipv6 = false;
+
+		// First peel off the socket parameter from the right, if it exists.
+		$socket_pos = strpos( $host, ':/' );
+		if ( $socket_pos !== false ) {
+			$socket = substr( $host, $socket_pos + 1 );
+			$host = substr( $host, 0, $socket_pos );
+		}
+
+		// We need to check for an IPv6 address first.
+		// An IPv6 address will always contain at least two colons.
+		if ( substr_count( $host, ':' ) > 1 ) {
+			$pattern = '#^(?:\[)?(?P<host>[0-9a-fA-F:]+)(?:\]:(?P<port>[\d]+))?#';
+			$is_ipv6 = true;
+		} else {
+			// We seem to be dealing with an IPv4 address.
+			$pattern = '#^(?P<host>[^:/]*)(?::(?P<port>[\d]+))?#';
+		}
+
+		$matches = array();
+		$result = preg_match( $pattern, $host, $matches );
+
+		if ( 1 !== $result ) {
+			// Couldn't parse the address, bail.
+			return false;
+		}
+
+		$host = '';
+		foreach ( array( 'host', 'port' ) as $component ) {
+			if ( ! empty( $matches[ $component ] ) ) {
+				$$component = $matches[ $component ];
+			}
+		}
+
+		return array( $host, $port, $socket, $is_ipv6 );
+	}
+
 	/**
 	 * Checks that the connection to the database is still up. If not, try to reconnect.
 	 *
@@ -1873,7 +1911,6 @@ class wpdb {
 	 *
 	 * @since 3.9.0
 	 *
-	 * @access private
 	 * @see wpdb::query()
 	 *
 	 * @param string $query The query to run.
@@ -1895,6 +1932,64 @@ class wpdb {
 		}
 	}
 
+	/**
+	 * Generates and returns a placeholder escape string for use in queries returned by ::prepare().
+	 *
+	 * @since 4.8.3
+	 *
+	 * @return string String to escape placeholders.
+	 */
+	public function placeholder_escape() {
+		static $placeholder;
+
+		if ( ! $placeholder ) {
+			// If ext/hash is not present, compat.php's hash_hmac() does not support sha256.
+			$algo = function_exists( 'hash' ) ? 'sha256' : 'sha1';
+			// Old WP installs may not have AUTH_SALT defined.
+			$salt = defined( 'AUTH_SALT' ) && AUTH_SALT ? AUTH_SALT : (string) rand();
+
+			$placeholder = '{' . hash_hmac( $algo, uniqid( $salt, true ), $salt ) . '}';
+		}
+
+		/*
+		 * Add the filter to remove the placeholder escaper. Uses priority 0, so that anything
+		 * else attached to this filter will recieve the query with the placeholder string removed.
+		 */
+		if ( ! has_filter( 'query', array( $this, 'remove_placeholder_escape' ) ) ) {
+			add_filter( 'query', array( $this, 'remove_placeholder_escape' ), 0 );
+		}
+
+		return $placeholder;
+	}
+
+	/**
+	 * Adds a placeholder escape string, to escape anything that resembles a printf() placeholder.
+	 *
+	 * @since 4.8.3
+	 *
+	 * @param string $query The query to escape.
+	 * @return string The query with the placeholder escape string inserted where necessary.
+	 */
+	public function add_placeholder_escape( $query ) {
+		/*
+		 * To prevent returning anything that even vaguely resembles a placeholder,
+		 * we clobber every % we can find.
+		 */
+		return str_replace( '%', $this->placeholder_escape(), $query );
+	}
+
+	/**
+	 * Removes the placeholder escape strings from a query.
+	 *
+	 * @since 4.8.3
+	 *
+	 * @param string $query The query from which the placeholder will be removed.
+	 * @return string The query with the placeholder removed.
+	 */
+	public function remove_placeholder_escape( $query ) {
+		return str_replace( $this->placeholder_escape(), '%', $query );
+	}
+
 	/**
 	 * Insert a row into a table.
 	 *
@@ -1950,7 +2045,6 @@ class wpdb {
 	 *
 	 * Runs an insert or replace query based on $type argument.
 	 *
-	 * @access private
 	 * @since 3.0.0
 	 * @see wpdb::prepare()
 	 * @see wpdb::$field_types
@@ -2133,7 +2227,6 @@ class wpdb {
 	 * stripped, then field processing is rejected and the query fails.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $table  Table name.
 	 * @param array  $data   Field/value pair.
@@ -2170,7 +2263,6 @@ class wpdb {
 	 * Prepares arrays of value/format pairs as passed to wpdb CRUD methods.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param array $data   Array of fields to values.
 	 * @param mixed $format Formats to be mapped to the values in $data.
@@ -2206,7 +2298,6 @@ class wpdb {
 	 * the wpdb::process_field_formats() method.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param array  $data  As it comes from the wpdb::process_field_formats() method.
 	 * @param string $table Table name.
@@ -2237,7 +2328,6 @@ class wpdb {
 	 * For string fields, record the maximum string length that field can safely save.
 	 *
 	 * @since 4.2.1
-	 * @access protected
 	 *
 	 * @param array  $data  As it comes from the wpdb::process_field_charsets() method.
 	 * @param string $table Table name.
@@ -2439,7 +2529,6 @@ class wpdb {
 	 * Retrieves the character set for the given table.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $table Table name.
 	 * @return string|WP_Error Table character set, WP_Error object if it couldn't be found.
@@ -2540,7 +2629,6 @@ class wpdb {
 	 * Retrieves the character set for the given column.
 	 *
 	 * @since 4.2.0
-	 * @access public
 	 *
 	 * @param string $table  Table name.
 	 * @param string $column Column name.
@@ -2605,7 +2693,6 @@ class wpdb {
 	 * The length may either be specified as a byte length or a character length.
 	 *
 	 * @since 4.2.1
-	 * @access public
 	 *
 	 * @param string $table  Table name.
 	 * @param string $column Column name.
@@ -2698,7 +2785,6 @@ class wpdb {
 	 * the search to finish as soon as it encounters a non-ASCII character.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $string String to check.
 	 * @return bool True if ASCII, false if not.
@@ -2719,7 +2805,6 @@ class wpdb {
 	 * Check if the query is accessing a collation considered safe on the current version of MySQL.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $query The query to check.
 	 * @return bool True if the collation is safe, false if it isn't.
@@ -2777,7 +2862,6 @@ class wpdb {
 	 * Strips any invalid characters based on value/charset pairs.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param array $data Array of value arrays. Each value array has the keys
 	 *                    'value' and 'charset'. An optional 'ascii' key can be
@@ -2938,7 +3022,6 @@ class wpdb {
 	 * Strips any invalid characters from the query.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $query Query to convert.
 	 * @return string|WP_Error The converted query, or a WP_Error object if the conversion fails.
@@ -2984,7 +3067,6 @@ class wpdb {
 	 * Strips any invalid characters from the string for a given table and column.
 	 *
 	 * @since 4.2.0
-	 * @access public
 	 *
 	 * @param string $table  Table name.
 	 * @param string $column Column name.
@@ -3025,7 +3107,6 @@ class wpdb {
 	 * Find the first table name referenced in a query.
 	 *
 	 * @since 4.2.0
-	 * @access protected
 	 *
 	 * @param string $query The query to search.
 	 * @return string|false $table The table name found, or false if a table couldn't be found.
@@ -3092,7 +3173,6 @@ class wpdb {
 	 *
 	 * @since 3.5.0
 	 *
-	 * @access protected
 	 */
 	protected function load_col_info() {
 		if ( $this->col_info )
@@ -3189,7 +3269,6 @@ class wpdb {
 	 * Closes the current database connection.
 	 *
 	 * @since 4.5.0
-	 * @access public
 	 *
 	 * @return bool True if the connection was successfully closed, false if it wasn't,
 	 *              or the connection doesn't exist.
@@ -3279,7 +3358,7 @@ class wpdb {
 	 *
 	 * @param string $db_cap The feature to check for. Accepts 'collation',
 	 *                       'group_concat', 'subqueries', 'set_charset',
-	 *                       or 'utf8mb4'.
+	 *                       'utf8mb4', or 'utf8mb4_520'.
 	 * @return int|false Whether the database feature is supported, false otherwise.
 	 */
 	public function has_cap( $db_cap ) {
